#ifndef GUARD_socket_h
#define GUARD_socket_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file socket.h
/// @brief Class for unix socket header file

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include <string>
#include <vector>

#include <glibmm.h>

class MainManager;

/// @brief Class for unix socket
class UnixSocket {
    public:
	UnixSocket();
	~UnixSocket();

	bool Connect();
	void Close();
	bool Listen();

	bool Send(const int&, const void*, const int&);
	/// @brief Send message to prepared socket
	/// @param message Message to send
	/// @param size Size of message
	/// @return Success of send
	bool Send(const void* message, const int& size)
	{
	    if(fd_ != -1)
		return Send(fd_, message, size);
	    else
		return false;	
	}
	bool Send(const int&, const std::string&);
	/// @brief Send message to prepared socket
	/// @param message Message to send in std::string
	/// @return Success of send
	bool Send(const std::string& message)
	{
	    if(fd_ != -1)
		return Send(fd_, message);
	    else
		return false;	
	}
	bool SendSignal(const int&);
	/// @brief Send integer 1 to prepared socket
	/// @return Success of send
	bool SendSignal()
	{
	    if(fd_ != -1)
		return SendSignal(fd_);
	    else
		return false;	
	}
	
	bool RecvSignal(const int&);
	/// @brief Receive and test received data is integer 1
	/// @return Success of receive
	bool RecvSignal()
	{
	    if(fd_ != -1)
		return RecvSignal(fd_);
	    else 
		return false;	
	}
	
	void Accept();

	/// @brief Copy received data to dest and empty received data container.
	/// @param dest Destination to copy received data
	void ReceivedData(std::vector<std::string>& dest)
	{
	    Glib::Mutex::Lock lock(acceptMutex_);
	    
	    dest = receivedData_;
	    
	    std::vector<std::string>().swap(receivedData_);
	}

	void SetMainManager(MainManager*);

    private:
	UnixSocket(const UnixSocket&);
	UnixSocket& operator=(const UnixSocket&);
	
	const int integerSize_;			///< Size of integer
	
	std::vector<std::string> receivedData_;	///< Container to store received data
	std::string socketPath_;		///< Socket path( tmpdir + PACKAGE_NAME + username )
	struct sockaddr_un servAddr_;		///< Address for connection
	int fd_;				///< Main socket
	bool inThread_;				///< Test is thread running
	bool stopThread_;			///< To stop thread loop

	Glib::Thread* acceptThread_;		///< Thread
	Glib::Mutex acceptMutex_;		///< Mutex for deal container
	Glib::Dispatcher signalReceived_;	///< Dispatcher for signal to MainManager

	MainManager* mainManager_;		///< Program control MainManager 

	void UnlinkSocketPath();
	void AcceptLoop();
	void DataReceived(const int&, const int&, char*, const int&);

};

#endif
