#ifndef GUARD_tag_mp3_h
#define GUARD_tag_mp3_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file tag_mp3.h
/// @brief Class for process mp3 tag header file.

#include <string>

#include "tag.h"

/// @brief Class for process mp3 tag.
/// @todo Consider change to use id3lib.
class MP3Tag : public Tag {
    public:
	MP3Tag(const std::string& filename);
	virtual ~MP3Tag();

    private:
	enum FileType {
	    CBR = 0,		///< Constant bitrate file.
	    VBR = -1,		///< Variable bitrate file.
	    XING_VBR = -2	///< Variable bitrate file with XING header.
	};
	
	std::string filename_;

	bool validFile_;
	
	struct id3_file* id3File_;
	struct id3_tag* id3Tag_;

	const long xingFlagFrames_;
	long xingFlags_;
	unsigned long xingFrames_;

	Glib::ustring ProcessFrame(const char* id);

	bool XingParse(struct mad_bitptr ptr, unsigned int bitlen);

	virtual void ProcessAll();
	virtual void ProcessYear();
	virtual void ProcessTrack();
	virtual void ProcessTitle();
	virtual void ProcessAlbum();
	virtual void ProcessGenre();
	virtual void ProcessArtist();
	virtual void ProcessComment();
	virtual void ProcessTotalTime();

};

#endif

