// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file preference.cc
/// @brief Class for user preference setting source file.

#include <fstream>
#include <cstdlib>

#include "../include/preference.h"
#include "../include/misc.h"

Preference::Preference()
: generalTrayIconAction_("fileinfo"),
audioDriver_("default"),
audioAlsa5Card_("0"),
audioAlsa5Dev_("0"),
audioAlsa9Dev_("default"),
audioOssDsp_("/dev/dsp"),
audioSunDev_("/dev/audio"),
audioAixsDev_("/dev/baud0/1"),
mixerDev_("/dev/mixer"),
efxStereoEnhanceAmount_(1.0),
efxRoomAmount_(0.4),
efxRoom2Amount_(0.1),
efxHarmonyAmount_(0.4),
efxRhythmeAmount_(1.0),
efxDistortionAmount_(7.0),
successLoadConfig_(false),
generalLeaveOnTrayWhenIconify_(false),
effect_(false),
efxChanged_(true),
efxBassBoost_(false),
efxBassBoost2_(false),
efxStereoEnhance_(false),
efxTrebleBoost_(false),
efxRoom_(false),
efxRoom2_(false),
efxHarmony_(false),
efxRhythme_(false),
efxDistortion_(false),
efxGraphicEQ_(false)
{
    efxGraphicEQGain_[0] = 0.0f;
    efxGraphicEQGain_[1] = 0.0f;
    efxGraphicEQGain_[2] = 0.0f;
    efxGraphicEQGain_[3] = 0.0f;
    efxGraphicEQGain_[4] = 0.0f;
    efxGraphicEQGain_[5] = 0.0f;
    efxGraphicEQGain_[6] = 0.0f;
    efxGraphicEQGain_[7] = 0.0f;
    efxGraphicEQGain_[8] = 0.0f;
    efxGraphicEQGain_[9] = 0.0f;
    efxGraphicEQGain_[10] = 0.0f;

    successLoadConfig_ = LoadConfig();
}

Preference::~Preference()
{
}

/// @brief Load config file.
bool Preference::LoadConfig()
{
    std::string prefFilename = Glib::get_home_dir();
    prefFilename += "/.sagua/config";

    std::map<std::string, std::string> configMap;
    if(ConfigfileToMap(prefFilename, configMap)) {

	AssignConfig(generalTrayIconAction_, "generaltrayiconaction", configMap);
	AssignConfig(generalLeaveOnTrayWhenIconify_, "generalleaveontraywheniconify", configMap);
	AssignConfig(audioDriver_, "audiodriver", configMap);
	AssignConfig(audioAlsa5Card_, "audioalsa5card", configMap);
	AssignConfig(audioAlsa5Dev_, "audioalsa5dev", configMap);
	AssignConfig(audioAlsa9Dev_, "audioalsa9dev", configMap);
	AssignConfig(audioOssDsp_, "audioossdsp", configMap);
	AssignConfig(audioSunDev_, "audiosundev", configMap);
	AssignConfig(audioAixsDev_, "audioaixsdev", configMap);
	AssignConfig(mixerDev_, "mixerdev", configMap);
	AssignConfig(efxStereoEnhanceAmount_, "efxstereoenhanceamount", configMap); 
	AssignConfig(efxRoomAmount_, "efxroomamount", configMap);
	AssignConfig(efxRoom2Amount_, "efxroom2amount", configMap);
	AssignConfig(efxHarmonyAmount_, "efxharmonyamount", configMap);
	AssignConfig(efxRhythmeAmount_, "efxrhythmeamount", configMap);
	AssignConfig(efxDistortionAmount_, "efxdistortionamount", configMap);
	AssignConfig(efxGraphicEQGain_[0], "efxgraphiceqgain50", configMap);
	AssignConfig(efxGraphicEQGain_[1], "efxgraphiceqgain100", configMap);
	AssignConfig(efxGraphicEQGain_[2], "efxgraphiceqgain200", configMap);
	AssignConfig(efxGraphicEQGain_[3], "efxgraphiceqgain400", configMap);
	AssignConfig(efxGraphicEQGain_[4], "efxgraphiceqgain800", configMap);
	AssignConfig(efxGraphicEQGain_[5], "efxgraphiceqgain1.5k", configMap);
	AssignConfig(efxGraphicEQGain_[6], "efxgraphiceqgain3k", configMap);
	AssignConfig(efxGraphicEQGain_[7], "efxgraphiceqgain5k", configMap);
	AssignConfig(efxGraphicEQGain_[8], "efxgraphiceqgain7k", configMap);
	AssignConfig(efxGraphicEQGain_[9], "efxgraphiceqgain10k", configMap);
	AssignConfig(efxGraphicEQGain_[10], "efxgraphiceqgain15k", configMap);
	AssignConfig(effect_, "effect", configMap); 
	AssignConfig(efxBassBoost_, "efxbassboost", configMap);
	AssignConfig(efxBassBoost2_, "efxbassboost2", configMap);
	AssignConfig(efxStereoEnhance_, "efxstereoenhance", configMap);
	AssignConfig(efxTrebleBoost_, "efxtrebleboost", configMap);
	AssignConfig(efxRoom_, "efxroom", configMap);
	AssignConfig(efxRoom2_, "efxroom2", configMap);
	AssignConfig(efxHarmony_, "efxharmony", configMap);
	AssignConfig(efxRhythme_, "efxrhythme", configMap);
	AssignConfig(efxDistortion_, "efxdistortion", configMap);
	AssignConfig(efxGraphicEQ_, "efxgraphiceq", configMap);

	return true;
	
    } else
	// else fail to load file to map
	return false;   
}

/// @brief Find key from configMap and assign to boolean destination.
/// @param dest Destination.
/// @param find Key.
/// @param configMap Container to find.
void Preference::AssignConfig(bool& dest, const std::string& find, const std::map<std::string, std::string>& configMap)
{
    std::map<std::string, std::string>::const_iterator i = configMap.find(find);

    if( i != configMap.end() ) {
	std::string value = i->second;	
	
	if(value.size() == 1) {
	    long int intValue = strtol( value.c_str(), NULL, 10 );

	    switch(intValue) {
		case 1:
		    dest = true;
		    break;

		case 0:
		    dest = false;
		    break;
		    
		default:
		    break;	         
	    }
	} else
	    return;    
    }
}

/// @brief Find key from configMap and assign to double destination.
/// @param dest Destination.
/// @param find Key.
/// @param configMap Container to find.
void Preference::AssignConfig(double& dest, const std::string& find, const std::map<std::string, std::string>& configMap)
{
    std::map<std::string, std::string>::const_iterator i = configMap.find(find);

    if( i != configMap.end() ) {
	dest = atof( (i->second).c_str() );
    }

}

/// @brief Find key from configMap and assign to float destination.
/// @param dest Destination.
/// @param find Key.
/// @param configMap Container to find.
void Preference::AssignConfig(float& dest, const std::string& find, const std::map<std::string, std::string>& configMap)
{
    std::map<std::string, std::string>::const_iterator i = configMap.find(find);

    if( i != configMap.end() ) {
	dest = atof( (i->second).c_str() );
    }

}

/// @brief Find key from configMap and assign to std::string destination.
/// @param dest Destination.
/// @param find Key.
/// @param configMap Container to find.
void Preference::AssignConfig(std::string& dest, const std::string& find, const std::map<std::string, std::string>& configMap)
{
    std::map<std::string, std::string>::const_iterator i = configMap.find(find);

    if( i != configMap.end() ) 
	dest = i->second;
}

